"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var typescript_object_utils_1 = require("typescript-object-utils");
var AnimationEngine_1 = require("../AnimationEngine");
var timingFunction_1 = require("../timingFunction");
var resolvePropAnimationMethods_1 = require("./resolvePropAnimationMethods");
var shouldInterpolateProp_1 = require("./shouldInterpolateProp");
var defaults = {
    duration: 1000,
    timingFunction: "linear",
    animationEngine: AnimationEngine_1.fallbackEngine
};
function animate(options) {
    options = options || defaults;
    var animationEngine = options.animationEngine || defaults.animationEngine;
    var hasPropOwnAnimationDefinition = resolvePropAnimationMethods_1.resolvePropAnimationMethods(options.properties);
    return function (WrappedComponent) {
        var Animate = (function (_super) {
            __extends(Animate, _super);
            function Animate(props) {
                var _this = _super.call(this, props) || this;
                _this.animationMap = {};
                var state = {};
                var keySource = hasPropOwnAnimationDefinition === null ? props : hasPropOwnAnimationDefinition;
                for (var key in keySource) {
                    if (keySource.hasOwnProperty(key)) {
                        _this.animationMap[key] = _this.getAnimationData(key);
                        state[key] = props[key];
                    }
                }
                _this.state = { props: state };
                return _this;
            }
            Animate.prototype.render = function () {
                return React.createElement(WrappedComponent, this.assembleProps());
            };
            Animate.prototype.componentWillUnmount = function () {
                for (var key in this.animationMap) {
                    if (this.animationMap.hasOwnProperty(key) && this.animationMap[key].stopAnimation) {
                        this.animationMap[key].stopAnimation();
                    }
                }
            };
            Animate.prototype.componentDidUpdate = function (prevProps) {
                if (typescript_object_utils_1.shallowEquals(prevProps, this.props))
                    return;
                for (var key in this.props) {
                    if (this.props.hasOwnProperty(key) && shouldInterpolateProp_1.shouldInterpolateProp(key, this.props, prevProps, hasPropOwnAnimationDefinition)) {
                        this.startPropAnimation(key);
                    }
                }
            };
            Animate.prototype.startPropAnimation = function (key) {
                var _this = this;
                var animationData = this.getAnimationData(key);
                if (animationData.stopAnimation)
                    animationData.stopAnimation();
                animationData.animationStartDate = Date.now();
                animationData.startValue = this.state.props[key];
                var tick = function () {
                    var diff = Date.now() - animationData.animationStartDate;
                    var a = Math.max(0, Math.min(1, diff / animationData.duration));
                    var timeCoefficient = animationData.timingFunction(a);
                    var prevProp = animationData.startValue;
                    var prop = _this.props[key];
                    var value = (prevProp) + (prop - prevProp) * timeCoefficient;
                    _this.setState({ props: typescript_object_utils_1.shallowMerge(_this.state.props, (_a = {}, _a[key] = value, _a)) });
                    return a !== 1;
                    var _a;
                };
                animationData.stopAnimation = animationEngine.startAnimation(tick);
            };
            Animate.prototype.getAnimationData = function (key) {
                if (!this.animationMap[key]) {
                    var config = typescript_object_utils_1.merge(defaults, options);
                    if (null !== hasPropOwnAnimationDefinition && hasPropOwnAnimationDefinition[key]) {
                        config = typescript_object_utils_1.merge(config, config.properties[key]);
                    }
                    this.animationMap[key] = {
                        timingFunction: timingFunction_1.timingFunction(config.timingFunction),
                        duration: config.duration,
                        startValue: this.props[key],
                        animationStartDate: 0,
                        stopAnimation: null
                    };
                }
                return this.animationMap[key];
            };
            Animate.prototype.assembleProps = function () {
                return typescript_object_utils_1.merge(this.props, this.state.props);
            };
            return Animate;
        }(React.Component));
        Animate.displayName = "Animate(" + (WrappedComponent["displayName"] || WrappedComponent["name"] || "Component") + ")";
        return Animate;
    };
}
exports.animate = animate;
